unit pivtools2;

interface

uses n2_matrix,n2_vector;

type tpeak=record
	   x,y:double; {peak location in the correlation plane}
	   diamx,diamy:double; {diameters of the peak in x and y direction}
	   cxy:double; {parameter with coupled x and y for 2D fit}
	   ph:double; {peak height}
	   valid:boolean;
	   end;
	 tcorr=record
	   c1,c2,c3:tmatrix; {c1 is without, c2 is with normalization, c3 is the normalization factor}
	   valid:boolean;
	   end;
	 tdisp=record
	   diam1,diam2:double;
	   dx1,dy1,dx2,dy2:double;
	   valid1,valid2:boolean;
	   corr:tcorr;
	   end;

function getairy(x,d:double):double;
function getairyc(x,d:double):double;
function getgauss(x,d:double):double;
function getcpeak1(a1,a2:tmatrix):tpeak; {2x 1D Gaussian fit, a1 to detect the maximum, a2 for sub-pixel location}
function getcpeak2(a1,a2:tmatrix):tpeak; {1x 2D Gaussian fit, a1 to detect the maximum, a2 for sub-pixel location}
function getcorrfft(a1,a2:tmatrix;win,norm:boolean):tcorr; {with FFT}
function getcorrzp(a1,a2:tmatrix;win,norm:boolean):tcorr; {with zero padding}
function getcorrln(a1,a2:tmatrix;win,norm:boolean):tcorr; {with local normalization}
function getdisp(a1,a2:tmatrix;methc,methp:byte;win,norm,secpeak:boolean):tdisp;

implementation

uses airy2,n2_mathe,n2_transforms,n2_complex;

function getairy(x,d:double):double; {d: diameter}
begin
  getairy:=fairy(abs(x/d)*7.66341194051355); {x=d/2 ==> getairy=0}
end;

function getairyc(x,d:double):double; {d: diameter}
begin
  getairyc:=getairy(mind(abs(0.5*d),abs(x)),d); {x>=d/2 ==> getairy=0}
end;

function getgauss(x,d:double):double; {d: diameter}
begin
  getgauss:=exp(-8*sqr(x/d)); {x=d/2 ==> getgauss=e^(-2)}
end;

function getcpeak1(a1,a2:tmatrix):tpeak; {correlation peak! (-size/2..+size/2; 0 at the center of the lower left pixel)}
var i,j,mi,mj:longint;
	mx,nx,ny,dx,dy,lnc,lnl,lnr,lnt,lnb:double;
begin
  getcpeak1.x:=0;
  getcpeak1.y:=0;
  getcpeak1.diamx:=0;
  getcpeak1.diamy:=0;
  getcpeak1.cxy:=0;
  getcpeak1.ph:=0;
  getcpeak1.valid:=false;
  if (length(a1)=length(a2)) and (length(a1[0])=length(a2[0])) then
  begin
	mi:=0;
	mj:=0;
	mx:=0;
	for j:=0 to length(a1)-1 do for i:=0 to length(a1[0])-1 do if a1[j,i]>mx then
	begin
	  mi:=i;
	  mj:=j;
	  mx:=a1[j,i]; {a1!}
	end;
	if (mx>0)  
	and (a2[mj,mi]>a2[mj,(mi+1) mod length(a2[0])]) and (a2[mj,mi]>a2[mj,(mi-1+length(a2[0])) mod length(a2[0])])
	and (a2[mj,mi]>a2[(mj+1) mod length(a2),mi]) and (a2[mj,mi]>a2[(mj-1+length(a2)) mod length(a2),mi])
	and (a2[mj,(mi+1) mod length(a2[0])]>0) and (a2[mj,(mi-1+length(a2[0])) mod length(a2[0])]>0)
	and (a2[(mj+1) mod length(a2),mi]>0) and (a2[(mj-1+length(a2)) mod length(a2),mi]>0) then
	begin
	  lnc:=ln(a2[mj,mi]);
	  lnl:=ln(a2[mj,(mi-1+length(a2[0])) mod length(a2[0])]);
	  lnr:=ln(a2[mj,(mi+1) mod length(a2[0])]);
	  lnb:=ln(a2[(mj-1+length(a2)) mod length(a2),mi]);
	  lnt:=ln(a2[(mj+1) mod length(a2),mi]);
	  nx:=lnl-2*lnc+lnr;
	  ny:=lnb-2*lnc+lnt;
	  if (nx<0) and (ny<0) then
	  begin
		dx:=mi+(lnl-lnr)/2/nx;
		dy:=mj+(lnb-lnt)/2/ny;
		getcpeak1.x:=dx-length(a2[0])*round(dx/length(a2[0]));
		getcpeak1.y:=dy-length(a2)*round(dy/length(a2));
		getcpeak1.ph:=exp(lnc-sqr(lnl-lnr)/8/nx-sqr(lnb-lnt)/8/ny);
		getcpeak1.diamx:=4/sqrt(-nx);
		getcpeak1.diamy:=4/sqrt(-ny);
		getcpeak1.valid:=true;
	  end;
	end;
  end;
end;

function getcpeak2(a1,a2:tmatrix):tpeak; {correlation peak! (-size/2..+size/2; 0 at the center of the lower left pixel)}
var i,j,mi,mj,ii,jj:longint;
	mx:double;
	lna:array[-1..1,-1..1] of double;
	c00,c10,c01,c20,c11,c02,dx,dy,nn:double;
begin
  getcpeak2.x:=0;
  getcpeak2.y:=0;
  getcpeak2.diamx:=0;
  getcpeak2.diamy:=0;
  getcpeak2.cxy:=0;
  getcpeak2.ph:=0;
  getcpeak2.valid:=false;
  if (length(a1)=length(a2)) and (length(a1[0])=length(a2[0])) then
  begin 
	mi:=0;
	mj:=0;
	mx:=0;
	for j:=0 to length(a1)-1 do for i:=0 to length(a1[0])-1 do if a1[j,i]>mx then
	begin
	  mi:=i;
	  mj:=j;
	  mx:=a1[j,i]; {a1!}
	end;
	if (mx>0) and (a2[(mj-1+length(a2)) mod length(a2),(mi-1+length(a2[0])) mod length(a2[0])]>0) and (a2[(mj-1+length(a2)) mod length(a2),mi]>0) and (a2[(mj-1+length(a2)) mod length(a2),(mi+1) mod length(a2[0])]>0) 
	and (a2[mj,(mi-1+length(a2[0])) mod length(a2[0])]>0) and (a2[mj,mi]>0) and (a2[mj,(mi+1) mod length(a2[0])]>0) 
	and (a2[(mj+1) mod length(a2),(mi-1+length(a2[0])) mod length(a2[0])]>0) and (a2[(mj+1) mod length(a2),mi]>0) and (a2[(mj+1) mod length(a2),(mi+1) mod length(a2[0])]>0) 
	then
	begin
	  lna[-1,-1]:=ln(a2[(mj-1+length(a2)) mod length(a2),(mi-1+length(a2[0])) mod length(a2[0])]);
	  lna[-1,0]:=ln(a2[(mj-1+length(a2)) mod length(a2),mi]);
	  lna[-1,1]:=ln(a2[(mj-1+length(a2)) mod length(a2),(mi+1) mod length(a2[0])]);
	  lna[0,-1]:=ln(a2[mj,(mi-1+length(a2[0])) mod length(a2[0])]);
	  lna[0,0]:=ln(a2[mj,mi]);
	  lna[0,1]:=ln(a2[mj,(mi+1) mod length(a2[0])]);
	  lna[1,-1]:=ln(a2[(mj+1) mod length(a2),(mi-1+length(a2[0])) mod length(a2[0])]);
	  lna[1,0]:=ln(a2[(mj+1) mod length(a2),mi]);
	  lna[1,1]:=ln(a2[(mj+1) mod length(a2),(mi+1) mod length(a2[0])]);
	  c20:=0;
	  c11:=0;
	  c02:=0;
	  for jj:=-1 to 1 do for ii:=-1 to 1 do 
	  begin
		c20:=c20+(3*sqr(jj)-2)*lna[jj,ii]/6;
		c02:=c02+(3*sqr(ii)-2)*lna[jj,ii]/6;
		c11:=c11+ii*jj*lna[jj,ii]/4;
	  end;
	  nn:=4*c02*c20-sqr(c11);
	  if (c20<0) and (c02<0) and (nn>0) then
	  begin
		c00:=0;
		c10:=0;
		c01:=0;
		for j:=-1 to 1 do for i:=-1 to 1 do 
		begin
		  c00:=c00+(5-3*sqr(i)-3*sqr(j))*lna[j,i]/9;
		  c10:=c10+j*lna[j,i]/6;
		  c01:=c01+i*lna[j,i]/6;
		end;
		dx:=(c11*c10-2*c01*c20)/nn;
		dy:=(c11*c01-2*c10*c02)/nn;
		if (abs(dx)<1) and (abs(dy)<1) then 
		begin
		  getcpeak2.x:=(mi+dx)-length(a2[0])*round((mi+dx)/length(a2[0]));
		  getcpeak2.y:=(mj+dy)-length(a2)*round((mj+dy)/length(a2));
		  getcpeak2.diamx:=sqrt(-8/c02);
		  getcpeak2.diamy:=sqrt(-8/c20);
		  getcpeak2.cxy:=c11;
		  getcpeak2.ph:=exp(c00-c02*sqr(dx)-c11*dx*dy-c20*sqr(dy));
		  getcpeak2.valid:=true;
		end;
	  end;
	end;
  end;
end;

function getcorrfft(a1,a2:tmatrix;win,norm:boolean):tcorr;
var c1,c2,o1,o2,o3:tmatrix;
	c1c,c2c:tmatrixc;
	i,j:longint;
begin
  getcorrfft.valid:=false;
  if (length(a1)=length(a2)) and (length(a1[0])=length(a2[0])) then
  begin  
	if norm then
	begin
	  o2:=initm(length(a1),length(a1[0]));
	  o3:=initm(length(a1),length(a1[0]));
	  if win then
	  begin
		c1:=initm(length(a1),length(a1[0]));
		for j:=0 to length(a1)-1 do for i:=0 to length(a1[0])-1 do c1[j,i]:=(1-abs((2*j+1)/length(a1)-1))*(1-abs((2*i+1)/length(a1[0])-1));
		c2:=initm(length(a2),length(a2[0]));
		for j:=0 to length(a2)-1 do for i:=0 to length(a2[0])-1 do c2[j,i]:=(1-abs((2*j+1)/length(a2)-1))*(1-abs((2*i+1)/length(a2[0])-1));
		c1c:=ft(c1);
		c2c:=ft(c2);
		for j:=0 to length(c1c)-1 do for i:=0 to length(c1c[0])-1 do c1c[j,i]:=mulc(conj(c1c[j,i]),c2c[j,i]);
		c1c:=ift(c1c);
		for j:=0 to length(o3)-1 do for i:=0 to length(o3[0])-1 do o3[j,i]:=c1c[j,i].re;
	  end
	  else
	  begin
		c1:=initm(length(a1),length(a1[0]));
		for j:=0 to length(a1)-1 do for i:=0 to length(a1[0])-1 do c1[j,i]:=1;
		c2:=initm(length(a2),length(a2[0]));
		for j:=0 to length(a2)-1 do for i:=0 to length(a2[0])-1 do c2[j,i]:=1;
		c1c:=ft(c1);
		c2c:=ft(c2);
		for j:=0 to length(c1c)-1 do for i:=0 to length(c1c[0])-1 do c1c[j,i]:=mulc(conj(c1c[j,i]),c2c[j,i]);
		c1c:=ift(c1c);
		for j:=0 to length(o3)-1 do for i:=0 to length(o3[0])-1 do o3[j,i]:=c1c[j,i].re;
	  end;
	end;
	if win then
	begin
	  c1:=initm(length(a1),length(a1[0]));
	  for j:=0 to length(a1)-1 do for i:=0 to length(a1[0])-1 do c1[j,i]:=a1[j,i]*(1-abs((2*j+1)/length(a1)-1))*(1-abs((2*i+1)/length(a1[0])-1));
	  c2:=initm(length(a2),length(a2[0]));
	  for j:=0 to length(a2)-1 do for i:=0 to length(a2[0])-1 do c2[j,i]:=a2[j,i]*(1-abs((2*j+1)/length(a2)-1))*(1-abs((2*i+1)/length(a2[0])-1));
	  c1c:=ft(c1);
	  c2c:=ft(c2);
	end
	else
	begin
	  c1c:=ft(a1);
	  c2c:=ft(a2);
	end;
	for j:=0 to length(c1c)-1 do for i:=0 to length(c1c[0])-1 do c1c[j,i]:=mulc(conj(c1c[j,i]),c2c[j,i]);
	c1c:=ift(c1c);
	o1:=initm(length(c1c),length(c1c[0]));
	for j:=0 to length(o1)-1 do for i:=0 to length(o1[0])-1 do o1[j,i]:=c1c[j,i].re;
	getcorrfft.c1:=o1;
	if norm then
	begin
	  for j:=0 to length(o2)-1 do for i:=0 to length(o2[0])-1 do o2[j,i]:=c1c[j,i].re/o3[j,i];
	  getcorrfft.c2:=o2;
	  getcorrfft.c3:=o3;
	end;
	getcorrfft.valid:=true;
  end
  else writeln('Only matrices of the same size for FFT.');
end;

function getcorrzp(a1,a2:tmatrix;win,norm:boolean):tcorr;
var c1,c2,o1,o2,o3:tmatrix;
	c1c,c2c:tmatrixc;
	i,j:longint;
begin
  getcorrzp.valid:=false;  
  if norm then
  begin
	o2:=initm(length(a1)+length(a2),length(a1[0])+length(a2[0]));
	o3:=initm(length(a1)+length(a2),length(a1[0])+length(a2[0]));
	if win then
	begin
	  c1:=initm(length(a1)+length(a2),length(a1[0])+length(a2[0]));
	  for j:=0 to length(a1)-1 do for i:=0 to length(a1[0])-1 do c1[j,i]:=(1-abs((2*j+1)/length(a1)-1))*(1-abs((2*i+1)/length(a1[0])-1));
	  c2:=initm(length(a1)+length(a2),length(a1[0])+length(a2[0]));
	  for j:=0 to length(a2)-1 do for i:=0 to length(a2[0])-1 do c2[j,i]:=(1-abs((2*j+1)/length(a2)-1))*(1-abs((2*i+1)/length(a2[0])-1));
	  c1c:=ft(c1);
	  c2c:=ft(c2);
	  for j:=0 to length(c1c)-1 do for i:=0 to length(c1c[0])-1 do c1c[j,i]:=mulc(conj(c1c[j,i]),c2c[j,i]);
	  c1c:=ift(c1c);
	  for j:=0 to length(o3)-1 do for i:=0 to length(o3[0])-1 do o3[j,i]:=c1c[j,i].re;
	end
	else
	begin
	  c1:=initm(length(a1)+length(a2),length(a1[0])+length(a2[0]));
	  for j:=0 to length(a1)-1 do for i:=0 to length(a1[0])-1 do c1[j,i]:=1;
	  c2:=initm(length(a1)+length(a2),length(a1[0])+length(a2[0]));
	  for j:=0 to length(a2)-1 do for i:=0 to length(a2[0])-1 do c2[j,i]:=1;
	  c1c:=ft(c1);
	  c2c:=ft(c2);
	  for j:=0 to length(c1c)-1 do for i:=0 to length(c1c[0])-1 do c1c[j,i]:=mulc(conj(c1c[j,i]),c2c[j,i]);
	  c1c:=ift(c1c);
	  for j:=0 to length(o3)-1 do for i:=0 to length(o3[0])-1 do o3[j,i]:=c1c[j,i].re;
	end;
  end;
  c1:=initm(length(a1)+length(a2),length(a1[0])+length(a2[0]));
  c2:=initm(length(a1)+length(a2),length(a1[0])+length(a2[0]));
  if win then
  begin
	for j:=0 to length(a1)-1 do for i:=0 to length(a1[0])-1 do c1[j,i]:=a1[j,i]*(1-abs((2*j+1)/length(a1)-1))*(1-abs((2*i+1)/length(a1[0])-1));
	for j:=0 to length(a2)-1 do for i:=0 to length(a2[0])-1 do c2[j,i]:=a2[j,i]*(1-abs((2*j+1)/length(a2)-1))*(1-abs((2*i+1)/length(a2[0])-1));
  end
  else
  begin
	for j:=0 to length(a1)-1 do for i:=0 to length(a1[0])-1 do c1[j,i]:=a1[j,i];
	for j:=0 to length(a2)-1 do for i:=0 to length(a2[0])-1 do c2[j,i]:=a2[j,i];
  end;
  c1c:=ft(c1);
  c2c:=ft(c2);
  for j:=0 to length(c1c)-1 do for i:=0 to length(c1c[0])-1 do c1c[j,i]:=mulc(conj(c1c[j,i]),c2c[j,i]);
  c1c:=ift(c1c);
  o1:=initm(length(c1c),length(c1c[0]));
  for j:=0 to length(o1)-1 do for i:=0 to length(o1[0])-1 do o1[j,i]:=c1c[j,i].re;
  getcorrzp.c1:=o1;
  if norm then
  begin
	for j:=0 to length(o2)-1 do for i:=0 to length(o2[0])-1 do if o3[j,i]>0 then o2[j,i]:=c1c[j,i].re/o3[j,i] else o2[j,i]:=0;
	getcorrzp.c2:=o2;
	getcorrzp.c3:=o3;
  end;
  getcorrzp.valid:=true;
end;

function getcorrln(a1,a2:tmatrix;win,norm:boolean):tcorr;
var c1,c2,c3,c4,c5,c6,c7,c8,c9:tmatrix;
	c1c,c2c,c3c,c4c,c5c,c6c:tmatrixc;
	i,j:longint;
begin
  getcorrln.valid:=false;
  c1:=initm(length(a1)+length(a2),length(a1[0])+length(a2[0]));
  c2:=initm(length(a1)+length(a2),length(a1[0])+length(a2[0]));
  c3:=initm(length(a1)+length(a2),length(a1[0])+length(a2[0]));
  c4:=initm(length(a1)+length(a2),length(a1[0])+length(a2[0]));
  c5:=initm(length(a1)+length(a2),length(a1[0])+length(a2[0]));
  c6:=initm(length(a1)+length(a2),length(a1[0])+length(a2[0]));
  if win then
  begin
	for j:=0 to length(a1)-1 do for i:=0 to length(a1[0])-1 do c1[j,i]:=(1-abs((2*j+1)/length(a1)-1))*(1-abs((2*i+1)/length(a1)-1))*a1[j,i];
	for j:=0 to length(a2)-1 do for i:=0 to length(a2[0])-1 do c2[j,i]:=(1-abs((2*j+1)/length(a2)-1))*(1-abs((2*i+1)/length(a2)-1))*a2[j,i];
	for j:=0 to length(a1)-1 do for i:=0 to length(a1[0])-1 do c3[j,i]:=(1-abs((2*j+1)/length(a1)-1))*(1-abs((2*i+1)/length(a1)-1))*sqr(a1[j,i]);
	for j:=0 to length(a2)-1 do for i:=0 to length(a2[0])-1 do c4[j,i]:=(1-abs((2*j+1)/length(a2)-1))*(1-abs((2*i+1)/length(a2)-1));
	for j:=0 to length(a1)-1 do for i:=0 to length(a1[0])-1 do c5[j,i]:=(1-abs((2*j+1)/length(a1)-1))*(1-abs((2*i+1)/length(a1)-1));
	for j:=0 to length(a2)-1 do for i:=0 to length(a2[0])-1 do c6[j,i]:=(1-abs((2*j+1)/length(a2)-1))*(1-abs((2*i+1)/length(a2)-1))*sqr(a2[j,i]);
  end
  else
  begin
	for j:=0 to length(a1)-1 do for i:=0 to length(a1[0])-1 do c1[j,i]:=a1[j,i];
	for j:=0 to length(a2)-1 do for i:=0 to length(a2[0])-1 do c2[j,i]:=a2[j,i];
	for j:=0 to length(a1)-1 do for i:=0 to length(a1[0])-1 do c3[j,i]:=sqr(a1[j,i]);
	for j:=0 to length(a2)-1 do for i:=0 to length(a2[0])-1 do c4[j,i]:=1;
	for j:=0 to length(a1)-1 do for i:=0 to length(a1[0])-1 do c5[j,i]:=1;
	for j:=0 to length(a2)-1 do for i:=0 to length(a2[0])-1 do c6[j,i]:=sqr(a2[j,i]);
  end;
  c1c:=ft(c1);
  c2c:=ft(c2);
  c3c:=ft(c3);
  c4c:=ft(c4);
  c5c:=ft(c5);
  c6c:=ft(c6);
  for j:=0 to length(c1c)-1 do for i:=0 to length(c1c[0])-1 do c1c[j,i]:=mulc(conj(c1c[j,i]),c2c[j,i]);
  for j:=0 to length(c3c)-1 do for i:=0 to length(c3c[0])-1 do c3c[j,i]:=mulc(conj(c3c[j,i]),c4c[j,i]);
  for j:=0 to length(c5c)-1 do for i:=0 to length(c5c[0])-1 do c5c[j,i]:=mulc(conj(c5c[j,i]),c6c[j,i]);
  c1c:=ift(c1c);
  c3c:=ift(c3c);
  c5c:=ift(c5c);
  c7:=initm(length(c1c),length(c1c[0]));
  c8:=initm(length(c1c),length(c1c[0]));
  c9:=initm(length(c1c),length(c1c[0]));
  for j:=0 to length(c7)-1 do for i:=0 to length(c7[0])-1 do c7[j,i]:=c1c[j,i].re;
  for j:=0 to length(c9)-1 do for i:=0 to length(c9[0])-1 do if c3c[j,i].re*c5c[j,i].re>0
	then c9[j,i]:=sqrt(c3c[j,i].re*c5c[j,i].re) 
	else c9[j,i]:=0;
  for j:=0 to length(c8)-1 do for i:=0 to length(c8[0])-1 do if c9[j,i]>0
	then c8[j,i]:=c1c[j,i].re/c9[j,i]
	else c8[j,i]:=0;
  getcorrln.c1:=c7;
  getcorrln.c2:=c8;
  getcorrln.c3:=c9;
  getcorrln.valid:=true;
end;

function getdisp(a1,a2:tmatrix;methc,methp:byte;win,norm,secpeak:boolean):tdisp;
var corr:tcorr;
	peak:tpeak;
	i,j:longint;
	dx,dy,vlu:double;
	c1,c2:tmatrix;
begin
  getdisp.dx1:=0;
  getdisp.dy1:=0;
  getdisp.dx2:=0;
  getdisp.dy2:=0;
  getdisp.valid1:=false;
  getdisp.valid2:=false;
  getdisp.diam1:=0;
  getdisp.diam2:=0;
  case methc of
  0: corr:=getcorrfft(a1,a2,win,norm);
  1: corr:=getcorrzp(a1,a2,win,norm);
  2: corr:=getcorrln(a1,a2,win,norm);
  end;
  if corr.valid then
  begin
	getdisp.corr:=corr;
	case methp of
    0: if norm then peak:=getcpeak1(corr.c1,corr.c2) else peak:=getcpeak1(corr.c1,corr.c1);
    1: if norm then peak:=getcpeak2(corr.c1,corr.c2) else peak:=getcpeak2(corr.c1,corr.c1);
    end;
	if peak.valid then
	begin
	  getdisp.dx1:=peak.x;
	  getdisp.dy1:=peak.y;
      getdisp.valid1:=true;
	  getdisp.diam1:=sqrt(sqr(peak.diamx)+sqr(peak.diamy));
	  if secpeak then
	  begin
		c1:=initm(length(corr.c1),length(corr.c1[0]));
        if norm then
	    begin
		  c2:=initm(length(corr.c2),length(corr.c2[0]));
	      for j:=0 to length(c2)-1 do for i:=0 to length(c2[0])-1 do 
	      begin
	        dx:=i-length(c2[0])*round(i/length(c2[0]))-peak.x;
	        dy:=j-length(c2)*round(j/length(c2))-peak.y;
		    case methp of
		    0: vlu:=peak.ph*exp(-8*sqr(dy/peak.diamy)-8*sqr(dx/peak.diamx));
		    1: vlu:=peak.ph*exp(-8*sqr(dy/peak.diamy)-8*sqr(dx/peak.diamx)+peak.cxy*dx*dy);
		    end;
	        c1[j,i]:=corr.c1[j,i]-vlu*corr.c3[j,i];
	        c2[j,i]:=corr.c2[j,i]-vlu;
	      end;
	    end
	    else
	    begin
	      for j:=0 to length(c1)-1 do for i:=0 to length(c1[0])-1 do 
	      begin
	        dx:=i-length(c1[0])*round(i/length(c1[0]))-peak.x;
	        dy:=j-length(c1)*round(j/length(c1))-peak.y;
		    case methp of
		    0: vlu:=peak.ph*exp(-8*sqr(dy/peak.diamy)-8*sqr(dx/peak.diamx));
		    1: vlu:=peak.ph*exp(-8*sqr(dy/peak.diamy)-8*sqr(dx/peak.diamx)+peak.cxy*dx*dy);
		    end;
	        c1[j,i]:=corr.c1[j,i]-vlu;
	      end;
	    end;
	    case methp of
        0: if norm then peak:=getcpeak1(c1,c2) else peak:=getcpeak1(c1,c1);
        1: if norm then peak:=getcpeak2(c1,c2) else peak:=getcpeak2(c1,c1);
        end;
	    if peak.valid then
	    begin
	      getdisp.dx2:=peak.x;
	      getdisp.dy2:=peak.y;
          getdisp.valid2:=true;
		  getdisp.diam2:=sqrt(sqr(peak.diamx)+sqr(peak.diamy));
		end;
	  end;
	end;
  end;
end;

begin
end.
